@@ -35,20 +35,21 @@ static void check_end (json_parse_t * parser)
     /* Our collection of bits and pieces. */	\
 						\
     json_parse_t parser_o = {0};		\
-    json_parse_t * parser = & parser_o
+    json_parse_t * parser = & parser_o;		\
+    json_parse_init (parser)
 
 #ifndef NOPERL
 
-/* Set up the parser. */
+/* Set up "parser" with the string from "json". */
 
-#define GETSTRING							\
-    {									\
-	STRLEN length;							\
-	parser->end = parser->input =					\
-	    (unsigned char *) SvPV (json, length);			\
-	parser->length = (unsigned int) length;				\
-	parser->unicode = SvUTF8 (json) ? 1 : 0;			\
-    }
+static void getstring (SV * json, json_parse_t * parser)
+{
+    STRLEN length;
+    parser->input = (unsigned char *) SvPV (json, length);
+    parser->end = parser->input;
+    parser->length = (unsigned int) length;
+    parser->unicode = SvUTF8 (json) ? 1 : 0;
+}
 
 #endif /* ndef NOPERL */
 
@@ -79,7 +80,7 @@ json_parse_run (json_parse_t * parser, SV * json)
 
     SV * r = & PL_sv_undef;
 
-    GETSTRING;
+    getstring (json, parser);
 
     if (parser->length == 0) {
 	fail_empty (parser);
@@ -92,10 +93,12 @@ json_parse_run (json_parse_t * parser, SV * json)
     switch (NEXTBYTE) {
 
     case '{':
+	INCDEPTH;
 	r = object (parser);
 	break;
 
     case '[':
+	INCDEPTH;
 	r = array (parser);
 	break;
 
@@ -196,10 +199,12 @@ c_validate (json_parse_t * parser)
     switch (NEXTBYTE) {
 
     case '{':
+	INCDEPTH;
 	valid_object (parser);
 	break;
 
     case '[':
+	INCDEPTH;
 	valid_array (parser);
 	break;
 
@@ -287,10 +292,6 @@ c_tokenize (json_parse_t * parser)
     }
 
     check_end (parser);
-#if 0
-    printf ("TOKENS:\n");
-    print_tokens (r);
-#endif /* 0 */
     return r;
 }
 
@@ -298,7 +299,6 @@ static void
 tokenize_free (json_token_t * token)
 {
     json_token_t * next;
-    static int nfree;
     next = token->child;
     if (next) {
 	if (! next->blessed) {
@@ -314,8 +314,6 @@ tokenize_free (json_token_t * token)
 	token->next = 0;
     }
     if (! token->blessed) {
-	//nfree++;
-	//fprintf (stderr, "Free %d %p\n", nfree, token);
 	Safefree (token);
     }
 }
@@ -327,7 +325,7 @@ validate (SV * json, unsigned int flags)
 {
     ENTRYDECL;
 
-    GETSTRING;
+    getstring (json, parser);
 
     if (parser->length == 0) {
 	fail_empty (parser);
@@ -338,7 +336,7 @@ validate (SV * json, unsigned int flags)
 static void
 check (json_parse_t * parser, SV * json)
 {
-    GETSTRING;
+    getstring (json, parser);
     c_validate (parser);
 }
 
@@ -347,7 +345,7 @@ tokenize (SV * json)
 {
     ENTRYDECL;
 
-    GETSTRING;
+    getstring (json, parser);
 
     /* Mark this parser as being used for tokenizing to bypass the
        checks for memory leaks when the parser is freed. */
